@@ -26,7 +26,7 @@
 #include "define.h"
 #include "sock.h"
 
-#define DEFAULT_PORT      5555
+#define JOIN_PORT         4649
 #define DEFAULT_TTL       64
 #define ALLOW_FIRST_NODE  1 //1,-1=True 0=False
 #define ID_DELETE         0
@@ -37,30 +37,30 @@
 #define DEFAULT_CACHE_DIR "cache"
 #define RECV_COUNT_MAX    10
 
-extern MChord   *pMChord;
-extern Protocol *pProtocol;
 
 
-MChord::MChord(int port, char *pcache_dir, int log_flg){
-     
+MChord::MChord(Protocol *pProtocol, int port, int sock, char *pcache_dir, 
+	       int log_flg, int Ip){
+  
   int n;
 
+  mProtocol = pProtocol;
+  mProtocol->SetMChord(this);
   mTaskList = NULL;
-
+  
   if(*pcache_dir == '\0'){
     strcpy(mCacheDir, DEFAULT_CACHE_DIR);
   }else{
     strcpy(mCacheDir, pcache_dir);
   }
 
-  if(port == 0){
-    mPort = DEFAULT_PORT;
-  }else{
-    mPort = port;
-  }
+  mPort = port;
+  mSock = sock;
+  
   fprintf(stderr, "ListenPort = %d\n", mPort);
   fprintf(stderr, "CacheDir = ./%s\n", mCacheDir);
   
+  
   //initialize random seed
   init_random();
   
@@ -75,7 +75,8 @@ MChord::MChord(int port, char *pcache_dir, int log_flg){
   mState = READY;
   
   //set MyIPAddr
-  mIp = str_to_ip(GetMyIp());
+  mIp = Ip;
+  //  mIp = str_to_ip(GetMyIp());
   fprintf(stderr, "IP = %s\n",GetMyIp());
 
   mNext.Ip   = mIp;
@@ -145,26 +146,50 @@ MChord::MChord(int port, char *pcache_dir, int log_flg){
 
 }
 
+
+void MChord::RecvPacket(int sock){
+
+  mProtocol->RecvPacket(sock);
+  
+}
+
+
 //Call by GUI
 //ex. Join("192.168.0.1:4649")
-void MChord::Join(char *addr){
+//ex. Join("127.0.0.1:4649")
+//ex. Join("localhost:4649")
+int MChord::Join(char *addr){
 
   char *port;
-  ADDRESS Address;  
-
+  ADDRESS Address;
+  char *ipstr;
+  char localhost[]="127.0.0.1";
+  
+  
+  
   port = strchr(addr,':');
   
   if(port == NULL){
-    Address.Port = DEFAULT_PORT;
+    Address.Port = JOIN_PORT;
   }else{
     *port = '\0';
     Address.Port = atoi(&port[1]);
   }
   
-  Address.Ip = str_to_ip(addr);
+  ipstr = host_to_ipstr(addr);
+  log(INFORMATION, "Join [%s]\n", ipstr);
+
+  Address.Ip = str_to_ip(ipstr);
+  if(str_to_ip(localhost) == Address.Ip){
+    mIp = Address.Ip;
+  }
   
   if(Address.Ip != mIp || Address.Port != mPort){
     SendJoin(Address);
+    return -1;//TRUE
+  }else{
+    return 0;//FALSE
+
   }
 }
 
@@ -184,6 +209,7 @@ void MChord::SendJoin(ADDRESS Address){
   mTmpSessId = sessid;
 
   //set task
+  mSendCount[F_JOIN] ++;
   ptask = SetTaskOfData(Address, M_JOIN, sessid, Address, 0, NONE, NONE);
   
   //send task
@@ -200,11 +226,12 @@ void MChord::RecvJoin(ADDRESS Address, int mcid, int sess_id, int seq_no){
   TASKLIST *ptask;
   ADDRESS AddressTo;
   ADDRESS AddressSrc;
-  
+  char localhost[]="127.0.0.1";
+
   mRecvCount[F_JOIN] ++;
 
   //NOT IMPLEMENTED
-  //if(mState != READY && (mState == NONE || ALLOW_FIRST_NODE )){
+  //if(mState != READY && (mState == STAND_ALONE || ALLOW_FIRST_NODE )){
   //    SendBusy();
   //    message("Receive Join : State is not READY\n");
   //    return; 
@@ -213,7 +240,14 @@ void MChord::RecvJoin(ADDRESS Address, int mcid, int sess_id, int seq_no){
 
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, seq_no);
+  mProtocol->SendAck(Address, seq_no);
+
+  //if receive packet from localhost[127.0.0.1]
+  //change mIp to 127.0.0.1
+  //for debug experiment
+  if(str_to_ip(localhost) == Address.Ip){
+    mIp = Address.Ip;
+  }
   
   message("begin=%u, mcid=%u, end=%u\n", mBeginId, mcid, mEndId);
   
@@ -243,7 +277,7 @@ void MChord::RecvJoin(ADDRESS Address, int mcid, int sess_id, int seq_no){
 	
 	message("mcid=%u, id=%u, mEndId=%u, sess_id=%d\n", 
 		mcid, idlist_local->id, mEndId, sess_id);
-	
+	mSendCount[F_FILE] ++;
 	SetTaskOfFile(Address, idlist_local->id, sess_id);
 	
       }
@@ -253,6 +287,7 @@ void MChord::RecvJoin(ADDRESS Address, int mcid, int sess_id, int seq_no){
     
     //Add queue with same sess_id.
     //Tell my EndId to the new node.
+    mSendCount[F_JOIN_ACCEPT] ++;
     SetTaskOfData(Address, M_JOIN_ACCEPT, sess_id, mNext, NONE, NONE, NONE);
     
     //Add callback task called after all task is done
@@ -280,11 +315,14 @@ void MChord::RecvJoin(ADDRESS Address, int mcid, int sess_id, int seq_no){
     AddressTo = LookupRT(mcid);
     AddressSrc.Ip   = mIp;
     AddressSrc.Port = mPort;
-
+    
+    
     //action = 3
-    recv_insert((RECVLIST **)&mRecvTaskList, sess_id, 3);
+    //set receive list to allow receive lookup response
+    recv_insert((RECVLIST **)&mRecvTaskList, sess_id, 3, NULL);
 
     //Lookup mcid
+    mSendCount[F_LOOKUP] ++;
     ptask = SetTaskOfData(AddressTo, M_LOOKUP, sess_id, AddressSrc, 
 			  mcid, NONE, NONE);
     SendTask(ptask);
@@ -337,7 +375,7 @@ void MChord::DeleteIdList(int mcid){
       }
       
       //return next list
-      pidlist = id_delete_key((IDLIST **)&mIdList, mcid);
+      pidlist = id_delete_key((IDLIST **)&mIdList, pidlist->id);
       id_print((IDLIST **)&mIdList);
 
     }else{
@@ -346,13 +384,15 @@ void MChord::DeleteIdList(int mcid){
     
   }
   
-
+  
   //recalc routing table, because last Id is change.
   //change STATE to READY
   if(mState == RING_UPDATING){
     mState = READY;
+    update_status("STATUS: Ready");
   }else{
     message("State CONNECTING Error\n"); 
+    update_status("STATUS: RING_UPDATING_ERROR");
   }
   
 }
@@ -499,7 +539,7 @@ void MChord::RecvFile(ADDRESS Address, int seq_no, int size, int sessid, int mci
   if(plist != NULL && plist->action == 2){
     log(NOTICE, "don't receive file on insert %d\n", plist->action);
     mSendCount[F_ACK] ++;
-    pProtocol->SendAck(Address, seq_no);      
+    mProtocol->SendAck(Address, seq_no);      
     return;
   }
 
@@ -522,7 +562,7 @@ void MChord::RecvFile(ADDRESS Address, int seq_no, int size, int sessid, int mci
       mFlag = 1;
       mFileSeqNo = fileseqno;
       mSendCount[F_ACK] ++;
-      pProtocol->SendAck(Address, seq_no);      
+      mProtocol->SendAck(Address, seq_no);      
       
     }else{
       log(WARNING, "state error mFlag = %d\n", mFlag);
@@ -536,7 +576,7 @@ void MChord::RecvFile(ADDRESS Address, int seq_no, int size, int sessid, int mci
     if(mFlag == 1 && ++ mFileSeqNo == fileseqno){
       fwrite(&buff[38], size - 16, 1, mFP);
       mSendCount[F_ACK] ++;
-      pProtocol->SendAck(Address, seq_no);
+      mProtocol->SendAck(Address, seq_no);
     }else{
       log(WARNING, "fileseqno error\n");
     }
@@ -555,7 +595,7 @@ void MChord::RecvFile(ADDRESS Address, int seq_no, int size, int sessid, int mci
       }
       mFlag = 0;
       mSendCount[F_ACK] ++;
-      pProtocol->SendAck(Address, seq_no);
+      mProtocol->SendAck(Address, seq_no);
     }else{
       log(WARNING, "error\n");
     }
@@ -582,7 +622,7 @@ void MChord::RecvFile(ADDRESS Address, int seq_no, int size, int sessid, int mci
 	id_insert((IDLIST **)&mIdList, mcid);
       }
       mSendCount[F_ACK] ++;
-      pProtocol->SendAck(Address, seq_no);
+      mProtocol->SendAck(Address, seq_no);
       
     }else{
       log(WARNING, "state error flag = %d\n", mFlag);  
@@ -598,8 +638,9 @@ void MChord::RecvFile(ADDRESS Address, int seq_no, int size, int sessid, int mci
 }
 
 //Call by Task
-//1뤫
-//ڤͤƺƷ׻
+//Call this method periodically. TODO
+//Call this method when receive JoinAccept
+//Call this method when finished sending join file (M_UPDATE_RT)
 void MChord::UpdateRT(){
 
   int cnt;
@@ -629,7 +670,6 @@ void MChord::UpdateRT(){
       //other range
       //set next ip
       
-      //TODO
       mRoutingTable[cnt].McId = mEndId + 1;
       mRoutingTable[cnt].Ip   = mNext.Ip;
       mRoutingTable[cnt].Port = mNext.Port;
@@ -643,14 +683,15 @@ void MChord::UpdateRT(){
 	    mIp, mPort);
   }
   
+  
   for(cnt = 0; cnt <= MCID_BIT; cnt++){
-
     sessid  = get_random();
-    recv_insert((RECVLIST **)&mRecvTaskList, sessid, 0);
-    SendLookupKey((int)pow(2,cnt) + mEndId, sessid);
+
+    //Send LookupResponse action = 0
+    SendLookupKey((int)pow(2,cnt) + mEndId, sessid, 0, NULL);
     
   }
-
+  
 }
 
 
@@ -660,12 +701,12 @@ void MChord::UpdateRT(){
 // Call by Protocol
 void MChord::RecvJoinAccept(ADDRESS Address, int SessId, ADDRESS Next,
 			    int EndId, int seq_no){
-
+  
   mRecvCount[F_JOIN_ACCEPT] ++;
-
+  
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, seq_no);
+  mProtocol->SendAck(Address, seq_no);
   
   //check the same node using bufferd sess_id
   if(SessId == mTmpSessId){
@@ -681,17 +722,27 @@ void MChord::RecvJoinAccept(ADDRESS Address, int SessId, ADDRESS Next,
   
   UpdateRT();
 
-  if(mState == NONE){
-    message("State = NONE\n");
+
+  if(mState == STAND_ALONE){
+    message("State = STAND_ALONE\n");
+    //update status on app
+    update_status("STATUS:none");
+
   }
   
   if(mState == CONNECTING){
     mState = READY;
     message("State = READY\n");
+    //update status on app
+    update_status("STATUS:Ready");
+
   }else{
     message("State RING_UPDATING Error\n");
+    //update status on app
+    //    update_status("STATUS:Ring Updating");
+
   }
-  
+
   message("write idlist\n");
   WriteFile("idlist.txt",(IDLIST **)&mIdList, mCacheDir);
 
@@ -703,7 +754,7 @@ void MChord::RecvJoinAccept(ADDRESS Address, int SessId, ADDRESS Next,
   Call by GUI
   for JOINROOM only
 */
-void MChord::SendLookupData(char *data){
+void MChord::SendLookupData(char *data, void(*callback)(int)){
   
   int key;
   int sessid;
@@ -712,31 +763,45 @@ void MChord::SendLookupData(char *data){
 
   message("Send Lookup Data key = %u\n", key);
   sessid  = get_random();
-  //JOINROOM = 1
-  recv_insert((RECVLIST **)&mRecvTaskList, sessid, 1);
+
   
-  SendLookupKey(key, sessid);
+  //action = 1
+  SendLookupKey(key, sessid, 1, callback);
 
 }
 
 
 //Call by GUI
-void MChord::SendLookupKey(int key, int sessid){
+void MChord::SendLookupKey(int key, int sess_id, int action, void(*callback)(int)){
   
-  ADDRESS Address;
-  ADDRESS AddressSrc;
+  ADDRESS  Address;
+  ADDRESS  AddressSrc;
   TASKLIST *ptask;
   
   Address = LookupRT(key);
-  AddressSrc.Ip = mIp;
-  AddressSrc.Port = mPort;
-
 
   message("Send Lookup Key Id = %u\n", key);
   print_ip(Address.Ip, Address.Port);
+
+
+  //if distination is me then SendFile
+  if(Address.Ip == mIp && Address.Port == mPort){
+    
+    Action(Address, key, key, sess_id, action);
+    
+  }else{
+    
+    //allow to receive LookupResponse
+    recv_insert((RECVLIST **)&mRecvTaskList, sess_id, action, callback);
+    AddressSrc.Ip   = mIp;
+    AddressSrc.Port = mPort;
+    mSendCount[F_LOOKUP] ++;
+    ptask = SetTaskOfData(Address, M_LOOKUP, sess_id, AddressSrc, key, 
+			  NONE, NONE);
+    SendTask(ptask);
+    
+  }
   
-  ptask = SetTaskOfData(Address, M_LOOKUP, sessid, AddressSrc, key, NONE, NONE);
-  SendTask(ptask);
   
 }
 
@@ -759,7 +824,7 @@ void MChord::RecvLookup(ADDRESS Address, int SessId, int McIdTo, int McIdFrom,
 
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, SeqNo);
+  mProtocol->SendAck(Address, SeqNo);
   
   idlist_local = mIdList;  
   send_flg = 0;  
@@ -771,17 +836,20 @@ void MChord::RecvLookup(ADDRESS Address, int SessId, int McIdTo, int McIdFrom,
     //my area. check id list
     while(idlist_local != NULL && send_flg == 0){
       if(idlist_local->id == McIdTo){
-	message("I have %u\n", McIdTo);
+	log(DEBUG, "I have %u\n", McIdTo);
 	if(AddressSrc.Ip == mIp && AddressSrc.Port == mPort){
-
-	  log(WARNING, "TODO: send me file\n");
+	  
+	  log(WARNING, "send me file\n");
 	}else{
 
 	  //send file
 	  message("send file\n");
+	  mSendCount[F_FILE] ++;
 	  SetTaskOfFile(AddressSrc, idlist_local->id, SessId);
 	}
+	
 	//return I have
+	mSendCount[F_LOOKUP] ++;
 	ptask = SetTaskOfData(AddressSrc, M_LOOKUP_RESPONSE, SessId, 
 			      AddressMe, McIdTo, FIND, Ttl);
 	
@@ -795,6 +863,7 @@ void MChord::RecvLookup(ADDRESS Address, int SessId, int McIdTo, int McIdFrom,
     //return I don't have
     if(send_flg == 0){
       message("I don't have %u\n", McIdTo);
+      mSendCount[F_LOOKUP] ++;
       ptask = SetTaskOfData(AddressSrc, M_LOOKUP_RESPONSE, SessId, 
 			    AddressMe, McIdTo, NOT_FIND, Ttl);
       SendTask(ptask);
@@ -813,7 +882,7 @@ void MChord::RecvLookup(ADDRESS Address, int SessId, int McIdTo, int McIdFrom,
     
     if(0 < Ttl){
       Address = LookupRT(McIdTo);
-      
+      mSendCount[F_LOOKUP] ++;
       ptask = SetTaskOfData(Address, M_LOOKUP, SessId, AddressSrc, McIdTo, 
 			    NONE, NONE);
       SendTask(ptask);
@@ -828,9 +897,7 @@ void MChord::RecvLookup(ADDRESS Address, int SessId, int McIdTo, int McIdFrom,
 //Call by Protocol
 void MChord::RecvLookupResponse(ADDRESS Address, int SessId, int McId,
 				int SeqNo, int Ans, int Ttl, int McId2){
-  
-  int n;
-  int sessid;
+
   RECVLIST *plist;
   
   mRecvCount[F_LOOKUP_RESPONSE] ++;
@@ -839,7 +906,7 @@ void MChord::RecvLookupResponse(ADDRESS Address, int SessId, int McId,
   
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, SeqNo);
+  mProtocol->SendAck(Address, SeqNo);
   
   //check sessid list is allowed
   plist = recv_search(mRecvTaskList, SessId);
@@ -850,10 +917,18 @@ void MChord::RecvLookupResponse(ADDRESS Address, int SessId, int McId,
     return;
   }
 
+  Action(Address, McId, McId2, SessId, plist->action);
+
   //delete sessid list
   recv_delete((RECVLIST **)&mRecvTaskList, plist); 
-  
-  switch(plist->action){
+
+}
+
+
+void MChord::Action(ADDRESS Address, int McId, int McId2, int SessId, int action){
+
+  int n;
+  switch(action){
 
   case 0:
     //Update RoutingTable if mcid matched
@@ -869,23 +944,20 @@ void MChord::RecvLookupResponse(ADDRESS Address, int SessId, int McId,
   case 1:
     //Login Lookup
     message("Recv Member\n");
+    
+    //call back application
     recv_lookup_response(McId);
     
   break;
 
   case 2:
     //Insert Lookup
-    //Send File
-    if(Address.Ip == mIp && Address.Port == mPort){
-      
-      log(WARNING, "send me file on insert\n");
-    }else{
-      
+    // Send File
+    if(Address.Ip != mIp || Address.Port != mPort){
       message("send file-----to port%d\n", Address.Port);
       SetTaskOfFile(Address, McId, SessId);
       SendTaskBySession(SessId);
     }
-
     break;
   
   case 3:
@@ -902,43 +974,28 @@ void MChord::RecvLookupResponse(ADDRESS Address, int SessId, int McId,
     break;
   }
 
-    
-  
-  
+
+
 }
-
-
-
 //Call by GUID
 void MChord::SendInsert(int key){
   
   ADDRESS Address;
   ADDRESS AddressSrc;
-  TASKLIST *ptask;  
-  int mcid;
+  TASKLIST *ptask;
+  
   int sess_id;
 
   //Join Room
-  if(mState != NONE && mState != CONNECTING){
-    
+  if(mState != STAND_ALONE && mState != CONNECTING){
+    mSendCount[F_INSERT] ++;    
     message("SendInsert Key=[%u]\n", key);
     
-    //Lookup RT
-    Address = LookupRT(key);
-    message("TTTTTTTTTTTTT%d[%d]\n",Address.Ip, Address.Port);
-    mSendCount[F_INSERT] ++;
-    
-    sess_id = get_random();
-
     //action = 2
-    recv_insert((RECVLIST **)&mRecvTaskList, sess_id, 2);
-    AddressSrc.Ip   = mIp;
-    AddressSrc.Port = mPort;
+    sess_id = get_random();
+    SendLookupKey(key, sess_id, 2, NULL);
     
-    ptask = SetTaskOfData(Address, M_LOOKUP, sess_id, AddressSrc, key, 
-			  NONE, NONE);
-    SendTask(ptask);
-
+  
   }else{
     log(WARNING, "Cannot Send while CONNECTING\n");
   }
@@ -957,27 +1014,28 @@ void MChord::RecvInsert(ADDRESS Address, char *data, int size, int mcid){
   
 }
 
-
 //Call by GUI
+void MChord::Leave(){
+
+  SendLeave();
+
+}
+
 void MChord::SendLeave(){
 
-  IDLIST *idlist_local;
+  IDLIST *idlist_local = mIdList;
   int sessid;
 
-  
-  mState    = DANGEROUS;
-
-  idlist_local = mIdList;
+  mState = DANGEROUS;
+  update_status("STATUS: Leaving");
   sessid = get_random();
 
-  //send to previd about leave from your node.
-  mSendCount[F_LEAVE] ++;
-  
   if(mPrev.Ip == mIp && mNext.Port == mPort){
     log(WARNING, "cannot send leave message to me.");
   }
-  
 
+  //send to prev id about leave from your node.
+  mSendCount[F_LEAVE] ++;
 
   //send leave
   //                            sessid, next,  mcid, ans,  ttl
@@ -1002,7 +1060,7 @@ void MChord::SendLeave(){
   SetTaskOfData(mPrev, M_LEAVE_END, sessid, mNext, NONE, NONE, NONE);
   
   SendTaskBySession(sessid);
-
+  mState = STAND_ALONE;
 }
 
 
@@ -1016,7 +1074,7 @@ void MChord::RecvLeave(ADDRESS Address, int sessid, ADDRESS next, int BeginId,
   
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, SeqNo);
+  mProtocol->SendAck(Address, SeqNo);
   
   
   //protocolnextɲäʤФʤʤ
@@ -1041,7 +1099,7 @@ void MChord::RecvLeaveEnd(ADDRESS Address, int sessid, int SeqNo){
 
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, SeqNo);
+  mProtocol->SendAck(Address, SeqNo);
   
   message("Recv Leave End sessid = %d\n", sessid);
 
@@ -1080,8 +1138,8 @@ ADDRESS MChord::LookupRT(int mcid){
     //message("|%d\n",mRoutingTable[cnt].Port);
     if((mRoutingTable[cnt].McId) <= mcid){
 
-    //if(( mEndId + pow(2, cnt) ) <= mcid){
-      ret = cnt;
+//if(( mEndId + pow(2, cnt) ) <= mcid){
+ret = cnt;
     }
 
   }
@@ -1105,9 +1163,9 @@ ADDRESS MChord::LookupRT(int mcid){
 
 TASKLIST *MChord::LookupTaskBySession(int sess_id ){
   
-  TASKLIST *ptask = mTaskList;
-  while(ptask != NULL && ptask->SessId != sess_id){
-    ptask = ptask->next;
+TASKLIST *ptask = mTaskList;
+while(ptask != NULL && ptask->SessId != sess_id){
+ptask = ptask->next;
   }
   
   return(ptask);
@@ -1129,13 +1187,13 @@ TASKLIST *MChord::SendTask(TASKLIST *task){
   case M_JOIN:
     message("M_JOIN sess_id = [%d]\n", task->SessId);
     mSendCount[F_JOIN] ++;
-    pProtocol->SendJoin(task->Address, task->SessId, mBeginId, task->SeqNo);
+    mProtocol->SendJoin(task->Address, task->SessId, mBeginId, task->SeqNo);
     break;
 
   case M_JOIN_ACCEPT:
     message("M_JOIN_ACCEPT\n");
     mSendCount[F_JOIN_ACCEPT] ++;
-    pProtocol->SendJoinAccept(task->Address, task->SessId, task->Next, mEndId, 
+    mProtocol->SendJoinAccept(task->Address, task->SessId, task->Next, mEndId, 
 			      task->SeqNo);
     break;
     
@@ -1153,21 +1211,21 @@ TASKLIST *MChord::SendTask(TASKLIST *task){
     mSendCount[F_FILE] ++;
     
     UpdateTaskOfFile(task);
-    pProtocol->SendFile(task->Address, task->Data, task->Size, task->SeqNo, 
+    mProtocol->SendFile(task->Address, task->Data, task->Size, task->SeqNo, 
 			task->SessId, task->McId, task->FileSeqNo, task->Flag);    
     break;
 
   case M_LOOKUP:
     message("M_LOOKUP\n");
     mSendCount[F_LOOKUP] ++;
-    pProtocol->SendLookup(task->Address, task->SessId, task->McId, mBeginId, 
+    mProtocol->SendLookup(task->Address, task->SessId, task->McId, mBeginId, 
 			  DEFAULT_TTL, task->SeqNo, task->Next);//Next=src
     break;
     
   case M_LOOKUP_RESPONSE:
     message("M_LOOKUP_RESPONSE\n");
     mSendCount[F_LOOKUP_RESPONSE] ++;
-    pProtocol->SendLookupResponse(task->Address, task->SessId, task->McId, 
+    mProtocol->SendLookupResponse(task->Address, task->SessId, task->McId, 
 				  task->SeqNo, task->Ttl, task->Ans, mEndId);
     break;
     
@@ -1175,7 +1233,7 @@ TASKLIST *MChord::SendTask(TASKLIST *task){
   case M_LEAVE:
     message("M_LEAVE sess_id = [%d]\n", task->SessId);
     mSendCount[F_LEAVE] ++;
-    pProtocol->SendLeave(task->Address, task->SessId, task->Next, mBeginId, 
+    mProtocol->SendLeave(task->Address, task->SessId, task->Next, mBeginId, 
 			 mEndId, task->SeqNo);
 
     break;
@@ -1183,7 +1241,7 @@ TASKLIST *MChord::SendTask(TASKLIST *task){
   case M_LEAVE_END:
     message("M_LEAVE_END sess_id = [%d]\n", task->SessId);
     mSendCount[F_LEAVE_END] ++;
-    pProtocol->SendLeaveEnd(task->Address, task->SessId, task->SeqNo);
+    mProtocol->SendLeaveEnd(task->Address, task->SessId, task->SeqNo);
     break;
 
 
@@ -1194,14 +1252,13 @@ TASKLIST *MChord::SendTask(TASKLIST *task){
     
   }
 
-  return task;
+return task;
 }
 
 
 
 
-TASKLIST *MChord::SetTaskOfData(ADDRESS Address, int Func, int SessId, ADDRESS Next,
-				int McId, int Ans, int Ttl){
+TASKLIST *MChord::SetTaskOfData(ADDRESS Address, int Func, int SessId, ADDRESS Next,int McId, int Ans, int Ttl){
 
   TASKLIST *ptask;
 
@@ -1324,9 +1381,9 @@ void MChord::ExecTask(){
   
 
   if(ptask == NULL){
-    message("TimerFunc : call by timer : No tasks\n");
+//    message("TimerFunc : call by timer : No tasks\n");
   }else{
-    message("TimerFunc : call by timer\n");   
+//    message("TimerFunc : call by timer\n");   
   }
 
   
@@ -1365,8 +1422,10 @@ void MChord::ExecTask(){
 
 
 /*
-  delete receive task.
-  call by TimerFunc();
+  
+  RecvTask is allow to receive packet of lookup response.
+  This function is delete receive task.
+  call by TimerFunc()
 
 */
 void MChord::DelRecvTask(){
@@ -1381,6 +1440,7 @@ void MChord::DelRecvTask(){
       
       //return next pointer
       ptask = recv_delete( (RECVLIST**)&mRecvTaskList, ptask);
+
       fprintf(stderr, "delete recvtask\n");
 
     }else{
@@ -1388,8 +1448,7 @@ void MChord::DelRecvTask(){
       //usually
       ptask = ptask->next;
     }
-
-   
+ 
   }
 
 
@@ -1510,7 +1569,7 @@ void MChord::PrintStatistics(void){
 
 #ifdef DEBUG
   return;
-#endif DEBUG
+#endif
 
   message("STATISTICS(Send/Recv)\n");
   
@@ -1556,6 +1615,10 @@ int MChord::GetLogFlg(){
   return mLogFlg;
 }
 
+int MChord::GetSock(){
+  return mSock;
+
+}
 
 //////////////////////
 //matchat extension///
@@ -1567,8 +1630,8 @@ void MChord::SendMessage(ADDRESS Address, char *data){
   int sessid;
 
   sessid = get_random();
-  
-  pProtocol->SendMessage(Address, sessid, data);
+
+  mProtocol->SendMessage(Address, sessid, data);
   
 }
 
@@ -1577,13 +1640,23 @@ void MChord::RecvMessage(ADDRESS Address, int SeqNo, char *data){
 
   //Send Ack
   mSendCount[F_ACK] ++;
-  pProtocol->SendAck(Address, SeqNo);
+  mProtocol->SendAck(Address, SeqNo);
   
   message("Recv Message %s\n",data);
   
-  //Call main
-  recv_data(data);
+  //call back application
+  //recv_data(data);
+  (*mRecvMessageFunc)(data);
 
+  
+}
+
+
+void MChord::SetCallBackRecvMessage(void(*recv_message_func)(char *)){
+
+  //set call back function
+  mRecvMessageFunc = recv_message_func;
+  
 }
 
 void MChord::message(char *fmt,...){
@@ -1596,7 +1669,7 @@ void MChord::message(char *fmt,...){
 #ifdef DEBUG
   //  fprintf(stderr, "%d:", mPort);
   //  vfprintf(stderr, fmt, ap);
-#endif DEBUG
+#endif
   
   if(mLogFlg == 1){ //1 set l flat
     fprintf(stderr, "%d:", mPort);
@@ -1617,6 +1690,20 @@ void MChord::message(char *fmt,...){
 
 }
 
+int MChord::IsJoin(){
+
+  if(mState == STAND_ALONE){
+
+    // false (alone)
+    return 0;
+
+  }else{
+
+    // true (join)
+    return -1;
+  }
+}
+
 void MChord::Quit(){
   
   //close(mLogFP);
