@@ -19,14 +19,16 @@ typedef struct _RECVLIST {
   int sess_id;
   int count;
   int action;
-
+  void(*func)(int);
+  
   struct _RECVLIST *prev;
   struct _RECVLIST *next;
 }RECVLIST;
 
 #endif
 
-RECVLIST *recv_insert(RECVLIST **phead, int sess_id, int action);
+RECVLIST *recv_insert(RECVLIST **phead, int sess_id, int action, 
+		      void(*func)(int));
 RECVLIST *recv_delete_key(RECVLIST **phead, int sess_id);
 RECVLIST *recv_delete(RECVLIST **phead, RECVLIST *pdel);
 
