@@ -13,3 +13,6 @@
 
 
 char *GetMyIp();
+char *host_to_ipstr(char *host);
+int  InitSocketTcp(int portno);
+int  InitSocketUdp(int portno);
